#include <formatio.h>
#include <userint.h>
#include "MainPanel.h"
#include "LL_3D_Scanner.h"
#include "Scan.h"
#include "GetDistance.h"

extern int 					MainPanel, MenuBar;
extern unsigned char 		curDir, lastLS, LaserCOM;
extern char 				directoryPath[600], fileName[600];
extern int  				FileHandle;   
extern const unsigned char 	UP, DOWN, STOPP, GO, ON, OFF;

double 						nextStopp;

double						DataZ[10000], DataY[10000];
int							DataPointer;

void ScanTIMER(unsigned char enable)
{
 SetCtrlAttribute (MainPanel, MainP_TIMER_Scan, ATTR_ENABLED, enable);
}


int CVICALLBACK SetZero_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 switch (event)
 {
  case EVENT_COMMIT:

  SetCtrlVal (MainPanel, MainP_currentPosition, 0.0);
  DataPointer = 0;
  DeleteGraphPlot (MainPanel, MainP_GRAPH, -1, VAL_IMMEDIATE_DRAW);
	  
  break;
 }
 return 0;
}


unsigned char WriteData(int X, double Y, double Z)
{
 char LineBuffer[80];
 
 FileHandle = OpenFile (fileName, VAL_WRITE_ONLY, VAL_APPEND, VAL_ASCII);	// open File
 
 Fmt(LineBuffer, "%d %f %f", X, Y, Z);
 WriteLine (FileHandle, LineBuffer, StringLength(LineBuffer));
 
 CloseFile (FileHandle);	
	
 return 0;
}


int CVICALLBACK Scan_start_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 int 	Station;
 char 	Message[80];
 double currentPosition, Step;  
 
 switch (event)
 {
  case EVENT_COMMIT:

  LineLaser(OFF);																// make sure line laser is turned off!
  SetMenuBarAttribute (MenuBar, MENUBAR_Tools_LineMarker,   ATTR_CHECKED, 0);	// clear check

  SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 1);
  SetCtrlAttribute (MainPanel, MainP_Scan_stopp,   ATTR_DIMMED, 0);
  
  SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 1);
  SetCtrlAttribute (MainPanel, MainP_STOPP_button, ATTR_DIMMED, 1);  
  
  SetCtrlVal (MainPanel, MainP_currentPosition, 0.0);							// set current position as zero

  GetCtrlVal (MainPanel, MainP_Station, &Station);								// generate file name
  Fmt(fileName, "%s\\%d.sldcrv", directoryPath, Station);						// using the station value
  FileHandle = OpenFile (fileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII);	// open File and dump whatever was in an maybe existing file before
  CloseFile (FileHandle);

  Fmt (Message, "\nFile name: %s", fileName); SetCtrlVal(MainPanel, MainP_Message, Message);

  SetCtrlVal (MainPanel, MainP_UpDone,  0);										// make sure ScanDone bits are cleared
  SetCtrlVal (MainPanel, MainP_LowDone, 0);   

  GetCtrlVal (MainPanel, MainP_currentPosition, &currentPosition);
  GetCtrlVal (MainPanel, MainP_Resolution, &Step);
  nextStopp = currentPosition + Step;

  DataPointer = 0;
  DeleteGraphPlot (MainPanel, MainP_GRAPH, -1, VAL_IMMEDIATE_DRAW);
  StoppMeasureGO(Station, 0);													// take initial sample at position = 0
  lastLS = LS();																// get current light barrier status
  Motor(UP);

  ScanTIMER(GO);

  break;
 }
 return 0;
}

int CVICALLBACK Scan_stopp_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 switch (event)
 {
  case EVENT_COMMIT:
	  
  Motor(STOPP);  	  
  ScanTIMER(STOPP);

  SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 0);
  SetCtrlAttribute (MainPanel, MainP_Scan_stopp,   ATTR_DIMMED, 1);	  

  SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 0);
  
  break;
 }
 return 0;
}


void StoppMeasureGO(int X, double Z)
{
 unsigned char saveDir = curDir;				// save current direction so that the motor can contuniue after the stopp into the same direction
 double		   Y;
 
 Motor(STOPP);
 Y = GetDistanceCOM(LaserCOM);
 WriteData(X, Y, Z);

 DataZ[DataPointer] = Z;
 DataY[DataPointer] = Y;
 PlotXY(MainPanel, MainP_GRAPH, DataY, DataZ, DataPointer+1, VAL_DOUBLE, VAL_DOUBLE, VAL_THIN_LINE, VAL_NO_POINT, VAL_SOLID, 1, VAL_YELLOW);
 DataPointer++;
 
 Motor(saveDir);
}


int CVICALLBACK TIMER_Scan_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 char			Message[80];
 double 		currentPosition, upperLimit, lowerLimit, Step, add = 0;
 int			Station, UpDone, LowDone;
 unsigned char	currentLS;		
	
 switch (event)
 {
  case EVENT_TIMER_TICK:

  GetCtrlVal (MainPanel, MainP_currentPosition, &currentPosition);
  GetCtrlVal (MainPanel, MainP_upperLimit, &upperLimit);
  GetCtrlVal (MainPanel, MainP_lowerLimit, &lowerLimit);  
  GetCtrlVal (MainPanel, MainP_Resolution, &Step);
  GetCtrlVal (MainPanel, MainP_Station, &Station);
  
  GetCtrlVal (MainPanel, MainP_UpDone,  &UpDone);
  GetCtrlVal (MainPanel, MainP_LowDone, &LowDone);
	  
  currentLS = LS();	  
  if (currentLS != lastLS) lastLS = currentLS;	// light sensor event, the sensor move by 0.5 mm; safe the current LS status
  else return 0;								// no event: just leave function

  if (curDir ==    UP) add =  0.5;				// update position
  if (curDir ==  DOWN) add = -0.5;
  if (curDir == STOPP) MessagePopup ("LS error", "The light sensor was triggert even after the motor status = STOPP!");
  currentPosition = currentPosition + add;
  SetCtrlVal (MainPanel, MainP_currentPosition, currentPosition);

  if (currentPosition == nextStopp)
  {
   StoppMeasureGO(Station, currentPosition);
   if (curDir == UP)   nextStopp = nextStopp + Step;
   if (curDir == DOWN) nextStopp = nextStopp - Step;
   Fmt (Message, "\nnextStopp = %f", nextStopp); SetCtrlVal(MainPanel, MainP_Message, Message);
  }
  
  if (currentPosition >= upperLimit)
  {
   UpDone = 1;
   SetCtrlVal (MainPanel, MainP_UpDone,  UpDone);
   nextStopp = currentPosition - Step;
   Fmt (Message, "  correction: nextStopp = %f", nextStopp); SetCtrlVal(MainPanel, MainP_Message, Message);   
   Motor(DOWN);
  }
  if (currentPosition <= lowerLimit)
  {
   LowDone = 1;
   SetCtrlVal (MainPanel, MainP_LowDone, LowDone);
   nextStopp = currentPosition + Step;
   Fmt (Message, "  correction: nextStopp = %f", nextStopp); SetCtrlVal(MainPanel, MainP_Message, Message);   
   Motor(UP);
  }

  if ((UpDone == 1) && (LowDone == 1) && (currentPosition >= 0))		// we drove up, down, up again and are now back in the starting position
  {
   Motor(STOPP);  	  
   ScanTIMER(STOPP);

   SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 0);
   SetCtrlAttribute (MainPanel, MainP_Scan_stopp,   ATTR_DIMMED, 1);	  
   SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 0);
  }

  break;
 }
 return 0;
}






























