// Code for handling the FT2232H in BitBang Mode
// autor: Dr. Marko Rocznik
// Date : 09.01.2015

#include <windows.h> 
#include <utility.h>
#include <formatio.h>
#include "ftd2xx.h"					// include FTDI device library
#include <userint.h>
#include "USBitBang.h"
#include "MainPanel.h"

extern MainPanel;


FT_HANDLE ftHandle_A, ftHandle_B;	// device Handles




void PopUp_error(unsigned long message)
{
 switch (message) 
 {
  case FT_OK	  						:  MessagePopup ("USB-Error", "alles OK :-)"); 				  break;    
  case FT_INVALID_HANDLE				:  MessagePopup ("USB-Error", "INVALID HANDLE"); 			  break;
  case FT_DEVICE_NOT_FOUND				:  MessagePopup ("USB-Error", "DEVICE NOT FOUND");			  break;
  case FT_DEVICE_NOT_OPENED				:  MessagePopup ("USB-Error", "DEVICE NOT OPENED");			  break;
  case FT_IO_ERROR						:  MessagePopup ("USB-Error", "IO-ERROR");					  break;
  case FT_INSUFFICIENT_RESOURCES		:  MessagePopup ("USB-Error", "INSUFFICIENT RESOURCES");	  break;
  case FT_INVALID_PARAMETER				:  MessagePopup ("USB-Error", "INVALID PARAMETER");			  break;
  case FT_INVALID_BAUD_RATE				:  MessagePopup ("USB-Error", "INVALID BAUD RATE");			  break;
  case FT_DEVICE_NOT_OPENED_FOR_ERASE   :  MessagePopup ("USB-Error", "DEVICE NOT OPENED FOR ERASE"); break;
  case FT_DEVICE_NOT_OPENED_FOR_WRITE   :  MessagePopup ("USB-Error", "DEVICE NOT OPENED FOR WRITE"); break;
  case FT_FAILED_TO_WRITE_DEVICE		:  MessagePopup ("USB-Error", "FAILED TO WRITE DEVICE");	  break;
  case FT_EEPROM_READ_FAILED			:  MessagePopup ("USB-Error", "EEPROM READ FAILED");		  break;
  case FT_EEPROM_WRITE_FAILED			:  MessagePopup ("USB-Error", "EEPROM WRITE FAILED");		  break;
  case FT_EEPROM_ERASE_FAILED			:  MessagePopup ("USB-Error", "EEPROM ERASE FAILED");		  break;
  case FT_EEPROM_NOT_PRESENT			:  MessagePopup ("USB-Error", "EEPROM NOT PRESENT");		  break;
  case FT_EEPROM_NOT_PROGRAMMED			:  MessagePopup ("USB-Error", "EEPROM NOT PROGRAMMED");		  break;
  case FT_INVALID_ARGS					:  MessagePopup ("USB-Error", "INVALID ARGS");				  break;
  case FT_NOT_SUPPORTED					:  MessagePopup ("USB-Error", "NOT_SUPPORTED");				  break;
  case FT_OTHER_ERROR					:  MessagePopup ("USB-Error", "OTHER_ERROR");				  break;
  default								:  ;														  break;
 }
}


/*--------------------------------------
// Open Device (set handles)
--------------------------------------*/
unsigned long open_USB_BitBang(char Device)
{
 FT_STATUS ftStatus;

 unsigned char DDRA, DDRB;			// data direction; 0 = input, 1 = output;
 
 DDRA = 0x0F;						// low nibble = output; high nibble = input;
 DDRB = 0x00;						// all inputs

//ftStatus =	FT_OK; 
 
 switch (Device)
 {
  case 'A' :	{
  				 ftStatus = FT_OpenEx("ProfileScaner A",FT_OPEN_BY_DESCRIPTION,&ftHandle_A);

  				 if (ftStatus != FT_OK) 
  				 { 
   				  PopUp_error(ftStatus);
                  return ftStatus;
  				 } 
				 
				 ftStatus = FT_SetBitMode(ftHandle_A,DDRA,FT_BITMODE_ASYNC_BITBANG);			// reset the I/O-Bit-Mode
				 
  				 if (ftStatus != FT_OK) 
  				 {
   				  PopUp_error(ftStatus);
   				  return ftStatus;
  				 }
				 break;
 				}
				
  case 'B' :	{
  				 ftStatus = FT_OpenEx("ProfileScaner B",FT_OPEN_BY_DESCRIPTION,&ftHandle_B);
				 
   				 if (ftStatus != FT_OK) 
  				 { 
   				  PopUp_error(ftStatus);
   				  return ftStatus;
  				 } 
				 
  				 ftStatus = FT_SetBitMode(ftHandle_B,DDRB,FT_BITMODE_ASYNC_BITBANG);			// reset the I/O-Bit-Mode
				  
  				 if (ftStatus != FT_OK) 
			     {
   				  PopUp_error(ftStatus);
   				  return ftStatus;
  				 } 
				 break;
 				}
  default:		{
	  			 MessagePopup ("USB-Error", "Can only open Device 'A' or Device 'B'");
				 return 100;
  				}
 }  

 return 0;
}


/*--------------------------------------
// Close Device
--------------------------------------*/
unsigned long close_USB(char Device)
{
 FT_STATUS ftStatus = 0;

 switch (Device)
 {
  case 'A' :  {ftStatus = FT_Close(ftHandle_A); break;}		// close Devices A
  case 'B' :  {ftStatus = FT_Close(ftHandle_B); break;}		// close Devices B
  default  :  {MessagePopup ("USB-Error", "Can only close Device 'A' or Device 'B'"); break;}
 }
  
 if (ftStatus != FT_OK) 
 {
  PopUp_error(ftStatus);
  return ftStatus;
 }

 return 0;
}

/*---------------------------------------------------------------------------------------------------
// write USB
---------------------------------------------------------------------------------------------------*/
unsigned char writeUSB(char Device, unsigned char word)
{
 FT_STATUS ftStatus;
 unsigned long written = 0;
 UCHAR TXbuf[1];				 
       TXbuf[0] = word;			  											

 ftStatus = 0;	   
	   
 switch (Device)
 {
	case 'A' : 	{ftStatus = FT_Write (ftHandle_A, &TXbuf, 1, &written); break;}						// Transmit this byte
	case 'B' :	{ftStatus = FT_Write (ftHandle_B, &TXbuf, 1, &written);	break;}						// Transmit this byte 
 }
	   
 if (ftStatus != FT_OK) 
 {
  PopUp_error(ftStatus);
  return (unsigned char) ftStatus;
 }
 if (written != 1) MessagePopup("write error", "unequal 1 byte written");

 return 0;
}


/*--------------------------------------
// Read USB
--------------------------------------*/
unsigned char readUSB(char Device)
{
 FT_HANDLE 				ftHandle;			// device handle
 UCHAR 					RXbuf[2];
 DWORD					RxBytes =0, TxBytes = 0, EventDWord;
 FT_STATUS 				ftStatus;
 unsigned long 			received = 0;

 ftHandle = ftHandle_A;	// inilize to prevent compiler warning
 switch (Device)
 {
  case 'A' : {ftHandle = ftHandle_A; break;};
  case 'B' : {ftHandle = ftHandle_B; break;};    
  default  : {MessagePopup ("USB-Error", "Can only close Device 'A' or Device 'B'"); break;}  
 }
 
 
 ftStatus = FT_GetStatus (ftHandle, &RxBytes, &TxBytes, &EventDWord);		// ask for the status; how many bytes are in the RX queue
 if (ftStatus != FT_OK) 													// check, if things are OK before reading
 {
  PopUp_error(ftStatus);
  return 0;
 }
  
 ftStatus = FT_Purge(ftHandle,FT_PURGE_RX | FT_PURGE_TX);					// clear device if there have been bytes
 ftStatus = FT_Read(ftHandle,RXbuf,1,&received);							// for some reason I have to perform one more read instruction, otherwise I read one old value before I get the updated one; make sure this works for your application!!
 ftStatus = FT_Purge(ftHandle,FT_PURGE_RX | FT_PURGE_TX);					// clear device if there have been bytes 

 ftStatus = FT_GetStatus (ftHandle, &RxBytes, &TxBytes, &EventDWord);
 RxBytes = 0;
 while (RxBytes == 0)														// wait until you receive at least one byte
 {
  ftStatus = FT_GetStatus (ftHandle, &RxBytes, &TxBytes, &EventDWord);
 }
 ftStatus = FT_Read(ftHandle,RXbuf,1,&received);							// read one byte  
 if (ftStatus != FT_OK) 
 {
  PopUp_error(ftStatus);
  return 0;
 }
 return RXbuf[0];															// return data value  
}
