#include <ansi_c.h>
#include <utility.h>
#include <userint.h>
#include "MainPanel.h"
#include "LL_3D_Scanner.h"

extern const unsigned char UP, DOWN, STOPP, GO, ON, OFF;
extern unsigned char curDir, lastLS;
extern int	MainPanel, MenuBar;

void manuTIMER(unsigned char enable)
{
 SetCtrlAttribute (MainPanel, MainP_TIMER, ATTR_ENABLED, enable);
}

int CVICALLBACK SetManualUpDown_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 double value, add = 0;
 
 switch (event)
 {
  case EVENT_COMMIT:

  GetCtrlVal (MainPanel, MainP_manualUpDownValue, &value);
  
  switch (control)
  {
   case MainP_down1mm	: {add = -  1; break;} 
   case MainP_down1cm	: {add = - 10; break;}
   case MainP_down10cm  : {add = -100; break;}
   case MainP_up1mm		: {add =    1; break;}
   case MainP_up1cm 	: {add =   10; break;}
   case MainP_up10cm    : {add =  100; break;}
   default				: {MessagePopup ("CB error", "The SetManualUpDown_CB was called from an invalid control"); break;}
  }
  
  SetCtrlVal (MainPanel, MainP_manualUpDownValue, value + add);
  
  break;
 }
 return 0;
}

int CVICALLBACK GO_button_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 double value;
 switch (event)
 {
  case EVENT_COMMIT:

  LineLaser(OFF);																// make sure line laser is turned off!
  SetMenuBarAttribute (MenuBar, MENUBAR_Tools_LineMarker,   ATTR_CHECKED, 0);	  
	  
  SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 1);
  SetCtrlAttribute (MainPanel, MainP_STOPP_button, ATTR_DIMMED, 0);
  
  SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 1);
  SetCtrlAttribute (MainPanel, MainP_Scan_stopp,   ATTR_DIMMED, 1);
  
  lastLS = LS();														// get current light barrier status
  GetCtrlVal (MainPanel, MainP_manualUpDownValue, &value);
  
  if (value > 0) { Motor(  UP); manuTIMER(GO); }
  if (value < 0) { Motor(DOWN); manuTIMER(GO); }
  if (value ==0)
  {
   Motor(STOPP);
   manuTIMER(STOPP);
   SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 0);
   SetCtrlAttribute (MainPanel, MainP_STOPP_button, ATTR_DIMMED, 1);
  }

  break;
 }
 return 0;
}

int CVICALLBACK STOPP_button_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 switch (event)
 {
  case EVENT_COMMIT:

  Motor(STOPP);
  manuTIMER(STOPP);
  SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 0);
  SetCtrlAttribute (MainPanel, MainP_STOPP_button, ATTR_DIMMED, 1);
  
  SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 0);
	  
  break;
 }
 return 0;
}

int CVICALLBACK TIMER_CB (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 unsigned char currentLS;
 double        value, add;
 
 switch (event)
 {
  case EVENT_TIMER_TICK:

  add = 0;						// preset: no event
	  
  currentLS = LS();	  
  if (currentLS != lastLS)		// light sensor event, the sensor move by 0.5 mm
  {
   if (curDir ==    UP) add = -0.5;
   if (curDir ==  DOWN) add =  0.5;
   if (curDir == STOPP) MessagePopup ("LS error", "The light sensor was triggert even after the motor status = STOPP!");
   
   lastLS = currentLS;			// safe the current LS status
  }
  else return 0;				// no event, just leave function
  

  GetCtrlVal (MainPanel, MainP_manualUpDownValue, &value);
  value = value + add;
  SetCtrlVal (MainPanel, MainP_manualUpDownValue, value);
  
  if (value > 0) if (curDir !=   UP) Motor(  UP);
  if (value < 0) if (curDir != DOWN) Motor(DOWN);
  if (value ==0 | fabs(value) < 0.5)
  {
   Motor(STOPP);
   manuTIMER(STOPP);
   SetCtrlVal (MainPanel, MainP_manualUpDownValue, 0.0);					// set value to zere on case that there was an non integer multiple of 0.5 entered
   SetCtrlAttribute (MainPanel, MainP_GO_button,    ATTR_DIMMED, 0);
   SetCtrlAttribute (MainPanel, MainP_STOPP_button, ATTR_DIMMED, 1);
   
   SetCtrlAttribute (MainPanel, MainP_Scan_start,   ATTR_DIMMED, 0);   
  }

  break;
 }
 return 0;
}


